//
//  ORAPDFTronBaseEditTool.swift
//  Aconex
//
//  Copyright © 2022 Oracle. All rights reserved.
//  Created by Rankit on 06/06/22.
//

import Foundation
import Tools

let ColorDidChange: NSNotification.Name = NSNotification.Name(rawValue: "ColorDidChange")

protocol BaseEditTool {
    func postColorChangeNotification(annot annotation: PTAnnot, annotStyle: PTAnnotStyle)
}

extension BaseEditTool {
    func postColorChangeNotification(annot annotation: PTAnnot, annotStyle: PTAnnotStyle) {
        NotificationCenter.default.post(name: ColorDidChange, object: self, userInfo: [
            "style": annotStyle,
            "type": annotation.extendedAnnotType,
        ])
    }
}

class AnnotEditTool: PTAnnotEditTool, BaseEditTool {
    override func annotStyleViewController(_ annotStyleViewController: PTAnnotStyleViewController, didChange annotStyle: PTAnnotStyle) {
        super.annotStyleViewController(annotStyleViewController, didChange: annotStyle)
        guard let annotation = self.currentAnnotation else { return }
        
        postColorChangeNotification(annot: annotation, annotStyle: annotStyle)
    }
    
}

class PolyAnnotEditTool: PTPolylineEditTool, BaseEditTool {
    override func annotStyleViewController(_ annotStyleViewController: PTAnnotStyleViewController, didChange annotStyle: PTAnnotStyle) {
        super.annotStyleViewController(annotStyleViewController, didChange: annotStyle)
        guard let annotation = self.currentAnnotation else { return }
                
        postColorChangeNotification(annot: annotation, annotStyle: annotStyle)

    }
}


class GroupToolbar: PTToolGroupToolbar {
    
    override init(frame: CGRect) {
        super.init(frame: frame)
    }
    
    override func setPresetsViewHidden(_ hidden: Bool, animated: Bool) {
        super.setPresetsViewHidden(true, animated: animated)
    }
    
    override init(toolGroupManager: PTToolGroupManager) {
        super.init(toolGroupManager: toolGroupManager)
        NotificationCenter.default.addObserver(self,
                                               selector: #selector(colorDidChange),
                                               name: ColorDidChange,
                                               object: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    deinit {
        NotificationCenter.default.removeObserver(self, name: ColorDidChange, object: nil)
    }
    
    @objc func colorDidChange(notification: Notification) {
         let userInfo = notification.userInfo!
         let style = userInfo["style"] as! PTAnnotStyle
         let type = userInfo["type"] as! PTExtendedAnnotType
         
         let presets = PTAnnotationStyleManager.default.stylePresets(forAnnotationType: type)
         var styles = presets.styles
         styles[Int(presets.selectedIndex)] = style
         presets.styles = styles
         PTAnnotationStyleManager.default.setStylePresets(presets, forAnnotationType: type)
         
         for item in self.items ?? [] {
             if let toolItem = item as? PTToolBarButtonItem {
                 if toolItem.toolClass?.annotType == type {
                     toolItem.setImageStyle(style)
                 }
             }
         }
     }
    
}
